const urlLocation = location.origin;
const API_URL =  `${urlLocation}/dat`;

const settingsPage = document.getElementById('settings-page');
const productsPage = document.getElementById('products-page');
const logsPage = document.getElementById('logs-page');

const saveButton = document.getElementById('save-button');
const retryButton = document.getElementById('retry-button');
const refreshButton = document.getElementById('refresh-button');
const pingDMButton = document.getElementById('ping-dm');
let taxCodesArray = [];
let paysCodesArray = [];
let taxesArray = [];

saveButton.addEventListener('click', saveSettings);
retryButton.addEventListener('click', saveSettings);

const connectionCheckboxes = document.getElementsByName('connection');

const getCurrentPageHash = () => {
    const pageHash = window.location.hash;

    let selectedTabId = '#settings';

    if (pageHash === '#products' || pageHash === '#logs') {
        selectedTabId = pageHash;
    }

    return selectedTabId;
}

const initiateTabs = () => {
    const tabs = document.querySelectorAll('.tab');
    const selectedTabId = getCurrentPageHash();

    showPageContent(selectedTabId);
    document.getElementById(selectedTabId).classList.add('active-tab');
    
    tabs.forEach(tab => tab.addEventListener('click', tabClickHandler));
    connectionCheckboxes.forEach(cb => cb.addEventListener('change', changeConnectionTypeHandler));
};

const tabClickHandler = ({ target }) => {
    const selectedTabId = target.getAttribute('id');
    const activeTab = document.querySelector('.tabs .active-tab');
    
    activeTab.classList.remove('active-tab');
    target.classList.add('active-tab');
    
    showPageContent(selectedTabId);
    window.history.pushState({}, '', selectedTabId);
};

const fillUpProductsTable = (goods) => {
    const productsTableBodyNode = document.getElementById('product-table-body');
    productsTableBodyNode.innerHTML = '';

    goods.forEach(({
        name,
        code,
        price,
        quantity,
        disc,
        taxgrp
    }) => {
        const tr = document.createElement('tr');

        tr.innerHTML = `
            <td>${code}</td>
            <td>${taxgrp}</td>
            <td>${name}</td>
            <td>${price}</td>
            <td>${quantity}</td>
            <td>${disc}</td>
        `;
    
        productsTableBodyNode.appendChild(tr);
    });
};

const changeConnectionTypeHandler = ({ target: { value } }) => {
    document.getElementById('com-port-settings').classList.add('hidden');
    document.getElementById('ethernet-port-settings').classList.add('hidden');

    if (value === 'ethernet') {    
        document.getElementById('ethernet-port-settings').classList.remove('hidden');
    }
    
    if (value === 'com') {
        document.getElementById('com-port-settings').classList.remove('hidden');
    }
};

const getPrintersAndPrrosFromDM = (dmLink) => {
    const dmErrorLink = document.getElementById('dm-error-link');
    const dmPrroList = document.getElementById('prro-names');
    const dmPrinterList = document.getElementById('printer-names');

    dmErrorLink.classList.add('hidden');
    dmPrroList.innerHTML = '';
    dmPrinterList.innerHTML = '';

    const DM_API_URL = dmLink === 'localhost' || dmLink === '127.0.0.1'
        ? 'http://127.0.0.1:3939'
        : `http://${dmLink}`;
    
    fetch(`${DM_API_URL}/dm/api/v3/devices`)
        .then(response => response.json())
        .then(({ devices }) => {
            const printerList = devices.filter(({ is_doc }) => is_doc);

            printerList.forEach(({ dev_code }) => {
                const option = document.createElement('option');
                option.value = dev_code;

                dmPrinterList.appendChild(option);
            });
        })
        .catch(() => {
            dmErrorLink.classList.remove('hidden');
            dmErrorLink.textContent = 'Помилка запиту, перевірте IP адресу ДМ';
        });

    fetch(`${DM_API_URL}/dm/executesql?ret_dataset=true`, {
        method: 'POST',
        body: 'A375515EF0FB2E0C7AE822B18EFE71E686FC8BD46E17AA25D76C4E4D6E6B4BCFC03977A24C3F62FD75CD900F885D52B79872A7FC384C5514B7E6E2F5BA897295F2CEFA9021836D44A089B646AED6849250DD18885640FA1E',
    })
    .then(response => response.json())
    .then(({ data: prros }) => {
        prros.forEach(({ device }) => {
            const option = document.createElement('option');
            option.value = device;

            dmPrroList.appendChild(option);
        })
    });
};

const showPageContent = (selectedTabId) => {
    settingsPage.classList.add('hidden');
    productsPage.classList.add('hidden');
    logsPage.classList.add('hidden');
    saveButton.classList.add('hidden');
    document.getElementById('save-message').classList.add('hidden');

    if (selectedTabId === '#settings') {
        showSettingsPage();
    }
    
    if (selectedTabId === '#products') {
        showProductsPage();
    } 
    
    if (selectedTabId === '#logs') {
        showLogsPage();
    }
};

const showSettingsPage = () => {
    settingsPage.classList.remove('hidden');
    saveButton.classList.remove('hidden');
    document.getElementById('com-port-settings').classList.add('hidden');
    document.getElementById('ethernet-port-settings').classList.add('hidden');

    const kryptonConnectionCheckbox = document.getElementById('krypton-protocol');
    const datecsConnectionCheckbox = document.getElementById('datecs-protocol');
    const comConnectionCheckbox = document.getElementById('com-connection');
    const ethernetConnectionCheckbox = document.getElementById('ethernet-connection');

    toggleLoader();
    fetch(`${API_URL}/getsettings`)
        .then(response => response.json())
        .then(({
            is_krypt_prot,
            is_com,
            vchasno_ip,
            is_print,
            is_plu_editable,
            debug_mode,
            disable_rounding,
            dev_sn,
            dev_model,
            logdays,
            archdays,
            vchasno_name,
            prn_name,
            com_port,
            com_port_sec,
            com_br,
            ethernet_port,
            taxCodesAssociated,
            paysCodesAssociated,
            taxes,
        }) => {
            if (is_krypt_prot) {
                kryptonConnectionCheckbox.checked = true
            } else {
                datecsConnectionCheckbox.checked = true;
            }

            if (is_com) {
                comConnectionCheckbox.checked = true;
                document.getElementById('com-port-settings').classList.remove('hidden');
            } else {
                ethernetConnectionCheckbox.checked = true;
                document.getElementById('ethernet-port-settings').classList.remove('hidden');
            }

            taxCodesArray = JSON.parse(JSON.stringify(taxCodesAssociated));
            createTableRows(taxCodesArray, "taxCodes");

            paysCodesArray = JSON.parse(JSON.stringify(paysCodesAssociated));
            createTableRows(paysCodesArray, "paysCodes");

            taxesArray = JSON.parse(JSON.stringify(taxes));
            createTableRows(taxesArray, "taxes");


            document.getElementById('dm-ip').value = vchasno_ip;
            document.getElementById('use-printer').checked = is_print;
            document.getElementById('edit-products').checked = is_plu_editable;
            document.getElementById('debug-mode').checked = debug_mode;
            document.getElementById('disable-rounding').checked = disable_rounding;
            document.getElementById('serial-number').value = dev_sn;
            document.getElementById('dev-model').value = dev_model;
            document.getElementById('logdays').value = logdays;
            document.getElementById('archdays').value = archdays;
            document.getElementById('prro-name').value = vchasno_name;
            document.getElementById('printer-name').value = prn_name;
            document.getElementById('com-port-speed').value = com_br;
            document.getElementById('com-port-name').value = com_port;
            document.getElementById('com-port-sec-name').value = com_port_sec;
            document.getElementById('ethernet-port-name').value = ethernet_port;
            toggleLoader();
            getPrintersAndPrrosFromDM(vchasno_ip);
        });
};

function createTableRows(dataArray, arrayType) {
    if (arrayType === "taxCodes") {
        const taxCodesBody = document.getElementById('taxCodesBody');
        taxCodesBody.innerHTML = '';

        dataArray.forEach((taxCode, index) => {
            const row = document.createElement('tr');

            // Create and append input field for prro_code
            const prroCodeInput = document.createElement('input');
            prroCodeInput.type = 'text';
            prroCodeInput.value = taxCode.prro_code;
            prroCodeInput.addEventListener('input', (event) => {
                dataArray[index].prro_code = parseInt(event.target.value) || 0;
            });
            const prroCodeCell = document.createElement('td');
            prroCodeCell.appendChild(prroCodeInput);
            row.appendChild(prroCodeCell);

            // Create and append input field for pos_code
            const posCodeInput = document.createElement('input');
            posCodeInput.type = 'text';
            posCodeInput.value = taxCode.pos_code;
            posCodeInput.addEventListener('input', (event) => {
                dataArray[index].pos_code = parseInt(event.target.value) || 0;
            });
            const posCodeCell = document.createElement('td');
            posCodeCell.appendChild(posCodeInput);
            row.appendChild(posCodeCell);

            // Append the row to the table body
            taxCodesBody.appendChild(row);
        });
    } else if (arrayType === "paysCodes") {
        const paysCodesBody = document.getElementById('paysCodesBody');
        paysCodesBody.innerHTML = '';

        dataArray.forEach((payCode, index) => {
            const row = document.createElement('tr');

            // Create and append input field for prro_code
            const prroCodeInput = document.createElement('input');
            prroCodeInput.type = 'text';
            prroCodeInput.value = payCode.prro_code;
            prroCodeInput.addEventListener('input', (event) => {
                dataArray[index].prro_code = parseInt(event.target.value) || 0;
            });
            const prroCodeCell = document.createElement('td');
            prroCodeCell.appendChild(prroCodeInput);
            row.appendChild(prroCodeCell);

            // Create and append input field for pos_code
            const posCodeInput = document.createElement('input');
            posCodeInput.type = 'text';
            posCodeInput.value = payCode.pos_code;
            posCodeInput.addEventListener('input', (event) => {
                dataArray[index].pos_code = parseInt(event.target.value) || 0;
            });
            const posCodeCell = document.createElement('td');
            posCodeCell.appendChild(posCodeInput);
            row.appendChild(posCodeCell);

            // Append the row to the table body
            paysCodesBody.appendChild(row);
        });
    } else {
        const taxesBody = document.getElementById('taxesBody');
        taxesBody.innerHTML = '';

        taxesArray.forEach((tax, index) => {
            const row = document.createElement('tr');

            // Create and append input field for id
            const idInput = document.createElement('input');
            idInput.type = 'text';
            idInput.value = tax.id;
            idInput.addEventListener('input', (event) => {
                taxesArray[index].id = parseInt(event.target.value) || 0;
            });
            const idCell = document.createElement('td');
            idCell.appendChild(idInput);
            row.appendChild(idCell);

            // Create and append input field for tax_type
            const taxTypeInput = document.createElement('input');
            taxTypeInput.type = 'text';
            taxTypeInput.value = tax.tax_type;
            taxTypeInput.addEventListener('input', (event) => {
                taxesArray[index].tax_type = event.target.value;
            });
            const taxTypeCell = document.createElement('td');
            taxTypeCell.appendChild(taxTypeInput);
            row.appendChild(taxTypeCell);

            // Create and append input field for tax_value
            const taxValueInput = document.createElement('input');
            taxValueInput.type = 'text';
            taxValueInput.value = tax.tax_value;
            taxValueInput.addEventListener('input', (event) => {
                taxesArray[index].tax_value = event.target.value;
            });
            const taxValueCell = document.createElement('td');
            taxValueCell.appendChild(taxValueInput);
            row.appendChild(taxValueCell);

            // Create and append input field for tax_name
            const taxNameInput = document.createElement('input');
            taxNameInput.type = 'text';
            taxNameInput.value = tax.tax_name;
            taxNameInput.addEventListener('input', (event) => {
                taxesArray[index].tax_name = event.target.value;
            });
            const taxNameCell = document.createElement('td');
            taxNameCell.appendChild(taxNameInput);
            row.appendChild(taxNameCell);

            // Create and append input field for tax_assoc
            const taxAssocInput = document.createElement('input');
            taxAssocInput.type = 'text';
            taxAssocInput.value = tax.tax_assoc;
            taxAssocInput.addEventListener('input', (event) => {
                taxesArray[index].tax_assoc = event.target.value;
            });
            const taxAssocCell = document.createElement('td');
            taxAssocCell.appendChild(taxAssocInput);
            row.appendChild(taxAssocCell);

            // Append the row to the table body
            taxesBody.appendChild(row);
        });
    }
}

const showProductsPage = () => {
    productsPage.classList.remove('hidden');

    fetch(`${API_URL}/goods`)
        .then(response => response.json())
        .then(({ goods }) => fillUpProductsTable(goods));    
};

const showLogsPage = () => {
    logsPage.classList.remove('hidden');

    const logsInfoNode = document.getElementById('logs-info');

    fetch(`${API_URL}/logs`)
        .then(response => response.arrayBuffer())
        .then(buffer => {
            const decoder = new TextDecoder('windows-1251');
            logsInfoNode.textContent = decoder.decode(buffer);
        })
        .catch(error => console.error(error));
};

refreshButton.addEventListener('click', () => {
    const selectedTabId = getCurrentPageHash();
    showPageContent(selectedTabId);
});

pingDMButton.addEventListener('click', () => {
    const dmLink = document.getElementById('dm-ip').value;
    getPrintersAndPrrosFromDM(dmLink);
});

function saveSettings(){
    const saveMessage = document.getElementById('save-message');
    saveMessage.innerHTML = '';
    saveMessage.classList.add('hidden');

    const is_krypt_prot = document.getElementById('krypton-protocol').checked;
    const is_com = document.getElementById('com-connection').checked;
    const com_port = document.getElementById('com-port-name').value;
    const com_port_sec = document.getElementById('com-port-sec-name').value;
    const com_br = document.getElementById('com-port-speed').value;
    const ethernet_port = document.getElementById('ethernet-port-name').value;
    const vchasno_ip = document.getElementById('dm-ip').value;
    const vchasno_name = document.getElementById('prro-name').value;
    const prn_name = document.getElementById('printer-name').value;
    const is_print = document.getElementById('use-printer').checked;
    const is_plu_editable = document.getElementById('edit-products').checked;
    const debug_mode = document.getElementById('debug-mode').checked;
    const disable_rounding = document.getElementById('disable-rounding').checked;
    const dev_sn = document.getElementById('serial-number').value;
    const dev_model = document.getElementById('dev-model').value;
    const logdays = document.getElementById('logdays').value;
    const archdays = document.getElementById('archdays').value;
    const taxCodesAssociated = taxCodesArray;
    const paysCodesAssociated = paysCodesArray;
    const taxes = taxesArray;
    toggleLoader();

    fetch(`${API_URL}/setsettings`, {
        method: 'POST',
        body: JSON.stringify({
            is_krypt_prot,
            is_com,
            com_port,
            com_port_sec,
            com_br: Number(com_br),
            ethernet_port: Number(ethernet_port),
            vchasno_ip,
            vchasno_name,
            prn_name,
            is_print,
            is_plu_editable,
            debug_mode,
            disable_rounding,
            dev_sn,
            dev_model,
            logdays,
            archdays,
            taxCodesAssociated,
            paysCodesAssociated,
            taxes,
        })
    })
    .then(response => response.json())
    .then(({ err_msg }) => {
        if (err_msg) {
            toggleLoader();
            showModalAsk(1, err_msg);
            console.log(err_msg);
        } else {
            toggleLoader();
            showModalAsk(0, err_msg);
            console.log(err_msg);
        }
    });
}

function showModalAsk(status, errorMessage) {
    if (status === 0) {
        document.getElementById('retry-button').classList.add('hidden');
        document.querySelector('.actionQuestion').innerHTML = `Налаштування збережено успішно`;
        document.getElementById('modal-ask-before-action').classList.remove('hidden');
        document.getElementById('overlay').classList.remove('hidden');
    } else {
        document.querySelector('.actionQuestion').innerHTML = `Помилка збереження налаштувань: ${errorMessage}`;
        document.getElementById('modal-ask-before-action').classList.remove('hidden');
        document.getElementById('overlay').classList.remove('hidden');
    }



}

function hideModalAsk() {
    document.getElementById('retry-button').classList.remove('hidden');
    document.getElementById('modal-ask-before-action').classList.add('hidden');
    document.getElementById('overlay').classList.add('hidden');
    const selectedTabId = getCurrentPageHash();
    showPageContent(selectedTabId);
}

function toggleLoader() {
    document.getElementById('loader-box').classList.toggle('active');
}

initiateTabs();